//===============================================================================
// @ IvMaterialOGL.cpp
// 
// Description
// ------------------------------------------------------------------------------
// Copyright (C) 2008  Elsevier, Inc.
//
// Implementation notes
//===============================================================================

//-------------------------------------------------------------------------------
//-- Dependencies ---------------------------------------------------------------
//-------------------------------------------------------------------------------

#include "IvMaterialOGL.h"
#include "IvVertexShaderOGL.h"
#include "IvFragmentShaderOGL.h"

//-------------------------------------------------------------------------------
//-- Static Members -------------------------------------------------------------
//-------------------------------------------------------------------------------

//-------------------------------------------------------------------------------
//-- Methods --------------------------------------------------------------------
//-------------------------------------------------------------------------------

//-------------------------------------------------------------------------------
// @ IvMaterialOGL::IvMaterialOGL()
//-------------------------------------------------------------------------------
// Default constructor
//-------------------------------------------------------------------------------
IvMaterialOGL::IvMaterialOGL() : IvMaterial(), mProgramID(0)
{
}

//-------------------------------------------------------------------------------
// @ IvMaterialOGL::~IvMaterialOGL()
//-------------------------------------------------------------------------------
// Destructor
//-------------------------------------------------------------------------------
IvMaterialOGL::~IvMaterialOGL()
{
    Destroy();
}

//-------------------------------------------------------------------------------
// @ IvMaterialOGL::CreateFromFile()
//-------------------------------------------------------------------------------
// Create a shader from a file
//-------------------------------------------------------------------------------
bool
IvMaterialOGL::Create( GLuint vertexShaderID, GLuint fragmentShaderID )
{
    // check for valid inputs
    if ( 0 == vertexShaderID || 0 == fragmentShaderID )
        return false;
    
    // allocate the shader id
    mProgramID = glCreateProgram();
    if ( mProgramID == 0 )
        return false;
    
    // attach the shaders
    glAttachShader( mProgramID, vertexShaderID );
    glAttachShader( mProgramID, fragmentShaderID );
    
    // link it
    glLinkProgram( mProgramID );
    GLint status = 0;
    glGetProgramiv( mProgramID, GL_LINK_STATUS, &status );
    if ( 0 == status )
        return false;
    
    return true;
}

//-------------------------------------------------------------------------------
// @ IvMaterialOGL::Destroy()
//-------------------------------------------------------------------------------
// Clean up before destructor
//-------------------------------------------------------------------------------
void
IvMaterialOGL::Destroy()
{
    if ( mProgramID != 0 )
    {
        glDeleteProgram( mProgramID );
    }
}
